@@ -1,12 +1,9 @@
-from typing import TYPE_CHECKING
+from typing import Final
 
-from ..types import TealType, require_type
-from ..ir import Op
-from .expr import Expr
-from .maybe import MaybeValue
-
-if TYPE_CHECKING:
-    from ..compiler import CompileOptions
+from pyteal.types import TealType, require_type
+from pyteal.ir import Op
+from pyteal.ast.expr import Expr
+from pyteal.ast.maybe import MaybeValue
 
 
 class AccountParam:
@@ -60,3 +57,33 @@ class AccountParam:
 
 
 AccountParam.__module__ = "pyteal"
+
+
+class AccountParamObject:
+    """Represents information about an account"""
+
+    def __init__(self, account: Expr) -> None:
+        """Create a new AccountParamObject for the given account.
+
+        Args:
+            account: An index into Txn.accounts that corresponds to the application to check or an
+                address available at runtime. May evaluate to uint64 or bytes, respectively.
+        """
+        self._account: Final = account
+
+    def balance(self) -> MaybeValue:
+        """Get the current balance in microAlgos for the account"""
+        return AccountParam.balance(self._account)
+
+    def min_balance(self) -> MaybeValue:
+        """Get the minimum balance in microAlgos for the account."""
+        return AccountParam.minBalance(self._account)
+
+    def auth_address(self) -> MaybeValue:
+        """Get the authorizing address for the account.
+
+        If the account is not rekeyed, the empty address is returned."""
+        return AccountParam.authAddr(self._account)
+
+
+AccountParamObject.__module__ = "pyteal"
