@@ -1,10 +1,9 @@
-from enum import Enum
+from typing import Final
 
-from ..types import TealType, require_type
-from ..ir import Op
-from .expr import Expr
-from .leafexpr import LeafExpr
-from .maybe import MaybeValue
+from pyteal.types import TealType, require_type
+from pyteal.ir import Op
+from pyteal.ast.expr import Expr
+from pyteal.ast.maybe import MaybeValue
 
 
 class AssetHolding:
@@ -17,7 +16,7 @@ class AssetHolding:
                 must be evaluated to uint64 (or, since v4, an account address that appears in
                 Txn.Accounts or is Txn.Sender, must be evaluated to bytes).
             asset: The ID of the asset to get, must be evaluated to uint64 (or, since v4,
-                a Txn.ForeignAssets offset).
+                a Txn.assets offset).
         """
         require_type(account, TealType.anytype)
         require_type(asset, TealType.uint64)
@@ -39,7 +38,7 @@ class AssetHolding:
                 must be evaluated to uint64 (or, since v4, an account address that appears in
                 Txn.Accounts or is Txn.Sender, must be evaluated to bytes).
             asset: The ID of the asset to get, must be evaluated to uint64 (or, since v4,
-                a Txn.ForeignAssets offset).
+                a Txn.assets offset).
         """
         require_type(account, TealType.anytype)
         require_type(asset, TealType.uint64)
@@ -54,15 +53,50 @@ class AssetHolding:
 AssetHolding.__module__ = "pyteal"
 
 
+class AssetHoldingObject:
+    """Represents information about an account's holding of an asset"""
+
+    def __init__(self, asset: Expr, account: Expr) -> None:
+        """Create a new AssetParamObject for the given asset.
+
+        Args:
+            asset: An identifier for the asset. It must be an index into Txn.ForeignAssets that
+                corresponds to the asset to check, or since v4, an asset ID that appears in
+                Txn.ForeignAssets. In either case, it must evaluate to uint64.
+            account: An identifier for the account. It must be an index into Txn.Accounts that
+                corresponds to the account to check (in which case it must evaluate to uint64), or
+                since v4, an account address that appears in Txn.Accounts or is Txn.Sender (in which
+                case it must evaluate to bytes).
+        """
+        require_type(asset, TealType.uint64)
+        self._asset: Final = asset
+        require_type(account, TealType.anytype)
+        self._account: Final = account
+
+    def balance(self) -> MaybeValue:
+        """Get the amount of the asset held by the account."""
+        return AssetHolding.balance(self._account, self._asset)
+
+    def frozen(self) -> MaybeValue:
+        """Check if the asset is frozen for the account.
+
+        A value of 1 indicates frozen and 0 indicates not frozen.
+        """
+        return AssetHolding.frozen(self._account, self._asset)
+
+
+AssetHoldingObject.__module__ = "pyteal"
+
+
 class AssetParam:
     @classmethod
     def total(cls, asset: Expr) -> MaybeValue:
         """Get the total number of units of an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -77,9 +111,9 @@ class AssetParam:
         """Get the number of decimals for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -94,9 +128,9 @@ class AssetParam:
         """Check if an asset is frozen by default.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -111,9 +145,9 @@ class AssetParam:
         """Get the unit name of an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -128,9 +162,9 @@ class AssetParam:
         """Get the name of an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -145,9 +179,9 @@ class AssetParam:
         """Get the URL of an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -164,9 +198,9 @@ class AssetParam:
         If set, this will be 32 bytes long.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -181,9 +215,9 @@ class AssetParam:
         """Get the manager address for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -198,9 +232,9 @@ class AssetParam:
         """Get the reserve address for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -215,9 +249,9 @@ class AssetParam:
         """Get the freeze address for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -232,9 +266,9 @@ class AssetParam:
         """Get the clawback address for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check,
+            asset: An index into Txn.assets that corresponds to the asset to check,
                 must be evaluated to uint64 (or since v4, an asset ID that appears in
-                Txn.ForeignAssets).
+                Txn.assets).
         """
         require_type(asset, TealType.uint64)
         return MaybeValue(
@@ -249,7 +283,7 @@ class AssetParam:
         """Get the creator address for an asset.
 
         Args:
-            asset: An index into Txn.ForeignAssets that corresponds to the asset to check. Must
+            asset: An index into Txn.assets that corresponds to the asset to check. Must
                 evaluate to uint64.
         """
         require_type(asset, TealType.uint64)
@@ -262,3 +296,71 @@ class AssetParam:
 
 
 AssetParam.__module__ = "pyteal"
+
+
+class AssetParamObject:
+    """Represents information about an asset's parameters"""
+
+    def __init__(self, asset: Expr) -> None:
+        """Create a new AssetParamObject for the given asset.
+
+        Args:
+            asset: An identifier for the asset. It must be an index into Txn.ForeignAssets that
+                corresponds to the asset to check, or since v4, an asset ID that appears in
+                Txn.ForeignAssets. In either case, it must evaluate to uint64.
+        """
+        require_type(asset, TealType.uint64)
+        self._asset: Final = asset
+
+    def total(self) -> MaybeValue:
+        """Get the total number of units of the asset."""
+        return AssetParam.total(self._asset)
+
+    def decimals(self) -> MaybeValue:
+        """Get the number of decimals for the asset."""
+        return AssetParam.decimals(self._asset)
+
+    def default_frozen(self) -> MaybeValue:
+        """Check if the asset is frozen by default."""
+        return AssetParam.defaultFrozen(self._asset)
+
+    def unit_name(self) -> MaybeValue:
+        """Get the unit name of the asset."""
+        return AssetParam.unitName(self._asset)
+
+    def name(self) -> MaybeValue:
+        """Get the name of the asset."""
+        return AssetParam.name(self._asset)
+
+    def url(self) -> MaybeValue:
+        """Get the URL of the asset."""
+        return AssetParam.url(self._asset)
+
+    def metadata_hash(self) -> MaybeValue:
+        """Get the arbitrary commitment for the asset.
+
+        If set, this will be 32 bytes long."""
+        return AssetParam.metadataHash(self._asset)
+
+    def manager_address(self) -> MaybeValue:
+        """Get the manager address for the asset."""
+        return AssetParam.manager(self._asset)
+
+    def reserve_address(self) -> MaybeValue:
+        """Get the reserve address for the asset."""
+        return AssetParam.reserve(self._asset)
+
+    def freeze_address(self) -> MaybeValue:
+        """Get the freeze address for the asset."""
+        return AssetParam.freeze(self._asset)
+
+    def clawback_address(self) -> MaybeValue:
+        """Get the clawback address for the asset."""
+        return AssetParam.clawback(self._asset)
+
+    def creator_address(self) -> MaybeValue:
+        """Get the creator address for the asset."""
+        return AssetParam.creator(self._asset)
+
+
+AssetParamObject.__module__ = "pyteal"
