@@ -1,13 +1,13 @@
 from typing import Union, cast, overload, TYPE_CHECKING
 
-from ..types import TealType, valid_base16, valid_base32, valid_base64
-from ..util import escapeStr
-from ..ir import TealOp, Op, TealBlock
-from ..errors import TealInputError
-from .leafexpr import LeafExpr
+from pyteal.types import TealType, valid_base16, valid_base32, valid_base64
+from pyteal.util import escapeStr
+from pyteal.ir import TealOp, Op, TealBlock
+from pyteal.errors import TealInputError
+from pyteal.ast.leafexpr import LeafExpr
 
 if TYPE_CHECKING:
-    from ..compiler import CompileOptions
+    from pyteal.compiler import CompileOptions
 
 
 class Bytes(LeafExpr):
@@ -15,14 +15,18 @@ class Bytes(LeafExpr):
 
     @overload
     def __init__(self, arg1: Union[str, bytes, bytearray]) -> None:
-        ...
+        pass
 
     @overload
     def __init__(self, arg1: str, arg2: str) -> None:
-        ...
+        pass
 
     def __init__(self, arg1: Union[str, bytes, bytearray], arg2: str = None) -> None:
-        """Create a new byte string.
+        """
+        __init__(arg1: Union[str, bytes, bytearray]) -> None
+        __init__(self, arg1: str, arg2: str) -> None
+
+        Create a new byte string.
 
         Depending on the encoding, there are different arguments to pass:
 
