@@ -1,15 +1,25 @@
 from enum import Enum
-from typing import TYPE_CHECKING, cast
 import algosdk
 
-from pyteal.ast.abi.util import type_specs_from_signature
+from typing import TYPE_CHECKING, cast
 from pyteal.ast.int import EnumInt
+from pyteal.ast.for_ import For
+from pyteal.ast.int import Int
+from pyteal.ast.scratchvar import ScratchVar
+
 from pyteal.ast.methodsig import MethodSignature
 from pyteal.types import TealType, require_type
 from pyteal.errors import TealInputError, TealTypeError, verifyProgramVersion
 from pyteal.ir import TealOp, Op, TealBlock
 from pyteal.ast.expr import Expr
-from pyteal.ast.txn import TxnField, TxnExprBuilder, TxnType, TxnaExprBuilder, TxnObject
+from pyteal.ast.txn import (
+    TxnType,
+    TxnArray,
+    TxnField,
+    TxnExprBuilder,
+    TxnaExprBuilder,
+    TxnObject,
+)
 from pyteal.ast.seq import Seq
 from pyteal.ast.bytes import Bytes
 from pyteal.ast import abi
@@ -149,6 +159,7 @@ class InnerTxnBuilder:
 
         Note: For non-array field (e.g., note), setting it twice will overwrite the original value.
               While for array field (e.g., accounts), setting it multiple times will append the values.
+              It is also possible to pass the entire array field if desired (e.g., Txn.accounts) to pass all the references.
 
         Requires program version 5 or higher. This operation is only permitted in application mode.
 
@@ -158,7 +169,7 @@ class InnerTxnBuilder:
                 compatible with the field being set.
         """
         if not field.is_array:
-            if type(value) is list:
+            if type(value) is list or isinstance(value, TxnArray):
                 raise TealInputError(
                     "inner transaction set field {} does not support array value".format(
                         field
@@ -166,25 +177,35 @@ class InnerTxnBuilder:
                 )
             return InnerTxnFieldExpr(field, cast(Expr, value))
         else:
-            if type(value) is not list:
+            if type(value) is not list and not isinstance(value, TxnArray):
                 raise TealInputError(
                     "inner transaction set array field {} with non-array value".format(
                         field
                     )
                 )
-            for valueIter in value:
-                if not isinstance(valueIter, Expr):
-                    raise TealInputError(
-                        "inner transaction set array field {} with non PyTeal expression array element {}".format(
-                            field, valueIter
+
+            if type(value) is list:
+                for valueIter in value:
+                    if not isinstance(valueIter, Expr):
+                        raise TealInputError(
+                            "inner transaction set array field {} with non PyTeal expression array element {}".format(
+                                field, valueIter
+                            )
                         )
-                    )
-            return Seq(
-                *[
-                    InnerTxnFieldExpr(field, cast(Expr, valueIter))
-                    for valueIter in value
-                ]
-            )
+
+                return Seq(
+                    *[
+                        InnerTxnFieldExpr(field, cast(Expr, valueIter))
+                        for valueIter in value
+                    ]
+                )
+            else:
+                arr = cast(TxnArray, value)
+                return For(
+                    (i := ScratchVar()).store(Int(0)),
+                    i.load() < arr.length(),
+                    i.store(i.load() + Int(1)),
+                ).Do(InnerTxnFieldExpr(field, arr[i.load()]))
 
     @classmethod
     def Execute(cls, fields: dict[TxnField, Expr | list[Expr]]) -> Expr:
@@ -196,7 +217,7 @@ class InnerTxnBuilder:
 
             InnerTxnBuilder.Begin()
             InnerTxnBuilder.SetFields(fields)
-            InnerTxnBuilder.End()
+            InnerTxnBuilder.Submit()
 
         Requires program version 5 or higher. This operation is only permitted in application mode.
 
@@ -216,6 +237,7 @@ class InnerTxnBuilder:
 
         Note: For non-array field (e.g., note), setting it twice will overwrite the original value.
               While for array field (e.g., accounts), setting it multiple times will append the values.
+              It is also possible to pass the entire array field if desired (e.g., Txn.accounts) to pass all the references.
 
         Requires program version 5 or higher. This operation is only permitted in application mode.
 
@@ -227,6 +249,63 @@ class InnerTxnBuilder:
         fieldsToSet = [cls.SetField(field, value) for field, value in fields.items()]
         return Seq(fieldsToSet)
 
+    @classmethod
+    def ExecuteMethodCall(
+        cls,
+        *,
+        app_id: Expr,
+        method_signature: str,
+        args: list[abi.BaseType | Expr | dict[TxnField, Expr | list[Expr]]],
+        extra_fields: dict[TxnField, Expr | list[Expr]] = None,
+    ) -> Expr:
+        """Performs a single app call transaction formatted as an ABI method call.
+
+        A convenience method that accepts fields to submit a single inner transaction, which is equivalent to:
+
+        .. code-block:: python
+
+            InnerTxnBuilder.Begin()
+            InnerTxnBuilder.MethodCall(
+                app_id=app_id,
+                method_signature=method_signature,
+                args=args,
+                extra_fields=extra_fields,
+            ),
+            InnerTxnBuilder.Submit()
+
+        Requires program version 5 or higher. This operation is only permitted in application mode.
+
+        Args:
+            app_id: An expression that evaluates to a `TealType.uint64` corresponding to the application being called.
+            method_signature: A string representing the method signature of the method we're calling. This is used to do
+                type checking on the arguments passed and to create the method selector passed as the first argument.
+            args: A list of arguments to pass to the application. The values in this list depend on the kind of argument you wish to pass:
+
+                - For basic ABI arguments (not Reference or Transaction types):
+                    If an ABI type is passed it **MUST** match the type specified in the `method_signature`. If an Expr is passed it must evaluate to `TealType.bytes` but beyond that no type checking is performed.
+
+                - For Reference arguments:
+                    Either the Reference type or an Expr that returns the type corresponding to the reference type are allowed.
+                    (i.e. Asset is TealType.uint64, Application is TealType.uint64, Account is TealType.bytes)
+
+                - For Transaction arguments:
+                    A dictionary containing TxnField to Expr that describe Transactions to be pre-pended to the transaction group being constructed.  The `TxnField.type_enum` key MUST be set and MUST match the expected transaction type specified in the `method_signature`.
+
+            extra_fields (optional): A dictionary whose keys are fields to set and whose values are the value each
+                field should take. Each value must evaluate to a type that is compatible with the
+                field being set. These fields are set on the ApplicationCallTransaction being constructed
+        """
+        return Seq(
+            cls.Begin(),
+            cls.MethodCall(
+                app_id=app_id,
+                method_signature=method_signature,
+                args=args,
+                extra_fields=extra_fields,
+            ),
+            cls.Submit(),
+        )
+
     @classmethod
     def MethodCall(
         cls,
@@ -234,7 +313,7 @@ class InnerTxnBuilder:
         app_id: Expr,
         method_signature: str,
         args: list[abi.BaseType | Expr | dict[TxnField, Expr | list[Expr]]],
-        extra_fields: dict[TxnField, Expr | list[Expr]] = {},
+        extra_fields: dict[TxnField, Expr | list[Expr]] = None,
     ) -> Expr:
         """Adds an ABI method call transaction to the current inner transaction group.
 
@@ -273,7 +352,7 @@ class InnerTxnBuilder:
 
         # We only care about the args
         arg_type_specs: list[abi.TypeSpec]
-        arg_type_specs, _ = type_specs_from_signature(method_signature)
+        arg_type_specs, _ = abi.type_specs_from_signature(method_signature)
 
         if len(args) != len(arg_type_specs):
             raise TealInputError(
@@ -403,7 +482,7 @@ class InnerTxnBuilder:
             # Set the fields for the app call in app args and foreign arrays
             *fields_to_set,
             # Add any remaining fields specified by the user
-            InnerTxnBuilder.SetFields(extra_fields),
+            InnerTxnBuilder.SetFields({} if extra_fields is None else extra_fields),
         )
 
 
