@@ -1,7 +1,7 @@
 import pytest
 
 import pyteal as pt
-from pyteal.ast.txn import TxnField, TxnType
+from pyteal.ast.txn import Txn, TxnField, TxnType
 from pyteal.types import types_match
 
 avm4Options = pt.CompileOptions(version=4)
@@ -111,6 +111,16 @@ ITXN_FIELDS_CASES = [
             ),
         ),
     ),
+    (
+        {pt.TxnField.accounts: pt.Txn.accounts},
+        pt.For(
+            (i := pt.ScratchVar()).store(pt.Int(0)),
+            i.load() < pt.Txn.accounts.length(),
+            i.store(i.load() + pt.Int(1)),
+        ).Do(
+            pt.InnerTxnBuilder.SetField(pt.TxnField.accounts, [Txn.accounts[i.load()]])
+        ),
+    ),
 ]
 
 
@@ -128,9 +138,14 @@ def test_InnerTxnBuilder_SetFields():
         actual.addIncoming()
         actual = pt.TealBlock.NormalizeBlocks(actual)
 
-        with pt.TealComponent.Context.ignoreExprEquality():
+        with pt.TealComponent.Context.ignoreScratchSlotEquality(), pt.TealComponent.Context.ignoreExprEquality():
             assert actual == expected
 
+        assert pt.TealBlock.MatchScratchSlotReferences(
+            pt.TealBlock.GetReferencedScratchSlots(actual),
+            pt.TealBlock.GetReferencedScratchSlots(expected),
+        )
+
         if len(fields) != 0:
             with pytest.raises(pt.TealInputError):
                 expr.__teal__(avm4Options)
@@ -152,9 +167,14 @@ def test_InnerTxnBuilder_Execute():
         actual.addIncoming()
         actual = pt.TealBlock.NormalizeBlocks(actual)
 
-        with pt.TealComponent.Context.ignoreExprEquality():
+        with pt.TealComponent.Context.ignoreScratchSlotEquality(), pt.TealComponent.Context.ignoreExprEquality():
             assert actual == expected
 
+        assert pt.TealBlock.MatchScratchSlotReferences(
+            pt.TealBlock.GetReferencedScratchSlots(actual),
+            pt.TealBlock.GetReferencedScratchSlots(expected),
+        )
+
         with pytest.raises(pt.TealInputError):
             expr.__teal__(avm4Options)
 
@@ -415,8 +435,16 @@ def test_InnerTxnBuilder_method_call(
                 args=args,
                 extra_fields=extra_fields,
             )
+        with pytest.raises(expected_error):
+            pt.InnerTxnBuilder.ExecuteMethodCall(
+                app_id=app_id,
+                method_signature=sig,
+                args=args,
+                extra_fields=extra_fields,
+            )
         return
 
+    # First run the test with MethodCall
     expr: pt.Expr = pt.InnerTxnBuilder.MethodCall(
         app_id=app_id, method_signature=sig, args=args, extra_fields=extra_fields
     )
@@ -438,241 +466,29 @@ def test_InnerTxnBuilder_method_call(
             pt.TealBlock.GetReferencedScratchSlots(expected),
         )
 
+    # Now run the same test with ExecuteMethodCall
+    expr = pt.InnerTxnBuilder.ExecuteMethodCall(
+        app_id=app_id, method_signature=sig, args=args, extra_fields=extra_fields
+    )
+    assert expr.type_of() == pt.TealType.none
+    assert not expr.has_return()
 
-ITXN_METHOD_CASES = (
-    (
-        pt.Int(1),
-        "add(uint64,uint64)void",
-        [t1_1 := pt.Itob(pt.Int(1)), t1_2 := pt.Itob(pt.Int(1))],
-        {TxnField.fee: pt.Int(0)},
-        pt.Seq(
-            pt.InnerTxnBuilder.SetFields(
-                {
-                    pt.TxnField.type_enum: TxnType.ApplicationCall,
-                    pt.TxnField.application_id: pt.Int(1),
-                    pt.TxnField.application_args: [
-                        pt.MethodSignature("add(uint64,uint64)void"),
-                        t1_1,
-                        t1_2,
-                    ],
-                    pt.TxnField.fee: pt.Int(0),
-                }
-            ),
-        ),
-        None,
-    ),
-    (
-        pt.Int(1),
-        "add(uint64,uint64)void",
-        [t2_1 := pt.abi.Uint64(), t2_2 := pt.abi.Uint64()],
-        {TxnField.fee: pt.Int(0)},
-        pt.Seq(
-            pt.InnerTxnBuilder.SetFields(
-                {
-                    pt.TxnField.type_enum: TxnType.ApplicationCall,
-                    pt.TxnField.application_id: pt.Int(1),
-                    pt.TxnField.application_args: [
-                        pt.MethodSignature("add(uint64,uint64)void"),
-                        t2_1.encode(),
-                        t2_2.encode(),
-                    ],
-                    pt.TxnField.fee: pt.Int(0),
-                }
-            ),
-        ),
-        None,
-    ),
-    (
-        pt.Int(1),
-        "add(application,account,asset)void",
-        [
-            t3_1 := pt.abi.Application(),
-            t3_2 := pt.abi.Account(),
-            t3_3 := pt.abi.Asset(),
-        ],
-        {TxnField.fee: pt.Int(0)},
-        pt.Seq(
-            pt.InnerTxnBuilder.SetFields(
-                {
-                    pt.TxnField.type_enum: TxnType.ApplicationCall,
-                    pt.TxnField.application_id: pt.Int(1),
-                    pt.TxnField.accounts: [t3_2.address()],
-                    pt.TxnField.applications: [t3_1.application_id()],
-                    pt.TxnField.assets: [t3_3.asset_id()],
-                    pt.TxnField.application_args: [
-                        pt.MethodSignature("add(application,account,asset)void"),
-                        pt.Bytes(b"\x01"),
-                        pt.Bytes(b"\x01"),
-                        pt.Bytes(b"\x00"),
-                    ],
-                    pt.TxnField.fee: pt.Int(0),
-                }
-            ),
-        ),
-        None,
-    ),
-    (
-        pt.Int(1),
-        "add(application,account,asset)void",
-        [
-            t4_1 := pt.Int(1),
-            t4_2 := pt.Global.zero_address(),
-            t4_3 := pt.Int(2),
-        ],
-        {TxnField.fee: pt.Int(0)},
-        pt.Seq(
-            pt.InnerTxnBuilder.SetFields(
-                {
-                    pt.TxnField.type_enum: TxnType.ApplicationCall,
-                    pt.TxnField.application_id: pt.Int(1),
-                    pt.TxnField.accounts: [t4_2],
-                    pt.TxnField.applications: [t4_1],
-                    pt.TxnField.assets: [t4_3],
-                    pt.TxnField.application_args: [
-                        pt.MethodSignature("add(application,account,asset)void"),
-                        pt.Bytes(b"\x01"),
-                        pt.Bytes(b"\x01"),
-                        pt.Bytes(b"\x00"),
-                    ],
-                    pt.TxnField.fee: pt.Int(0),
-                }
-            ),
-        ),
-        None,
-    ),
-    (
-        pt.Int(1),
-        "add(pay,txn,appl)void",
-        [
-            t5_1 := {TxnField.type_enum: TxnType.Payment},
-            t5_2 := {TxnField.type_enum: TxnType.AssetTransfer},
-            t5_3 := {TxnField.type_enum: TxnType.ApplicationCall},
-        ],
-        {TxnField.fee: pt.Int(0)},
-        pt.Seq(
-            pt.InnerTxnBuilder.SetFields(t5_1),  # type: ignore
-            pt.InnerTxnBuilder.Next(),
-            pt.InnerTxnBuilder.SetFields(t5_2),  # type: ignore
-            pt.InnerTxnBuilder.Next(),
-            pt.InnerTxnBuilder.SetFields(t5_3),  # type: ignore
-            pt.InnerTxnBuilder.Next(),
-            pt.InnerTxnBuilder.SetFields(
-                {
-                    pt.TxnField.type_enum: TxnType.ApplicationCall,
-                    pt.TxnField.application_id: pt.Int(1),
-                    pt.TxnField.application_args: [
-                        pt.MethodSignature("add(pay,txn,appl)void"),
-                    ],
-                    pt.TxnField.fee: pt.Int(0),
-                }
-            ),
-        ),
-        None,
-    ),
-    # Error cases
-    (
-        pt.Int(1),
-        "add(pay,txn,appl)void",
-        [
-            {},
-            {TxnField.type_enum: TxnType.AssetTransfer},
-            {TxnField.type_enum: TxnType.ApplicationCall},
-        ],
-        None,
-        None,
-        pt.TealInputError,
-    ),
-    (
-        pt.Int(1),
-        "add(pay,txn,appl)void",
-        [
-            {TxnField.type_enum: pt.Int(10)},
-            {TxnField.type_enum: TxnType.AssetTransfer},
-            {TxnField.type_enum: TxnType.ApplicationCall},
-        ],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(pay,txn,appl)void",
-        [
-            {TxnField.type_enum: TxnType.ApplicationCall},
-            {TxnField.type_enum: TxnType.AssetTransfer},
-            {TxnField.type_enum: TxnType.ApplicationCall},
-        ],
-        None,
-        None,
-        pt.TealInputError,
-    ),
-    (
-        pt.Int(1),
-        "add(application,account,asset)void",
-        [
-            pt.abi.Asset(),
-            pt.abi.Account(),
-            pt.abi.Asset(),
-        ],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(application)void",
-        [
-            pt.Bytes(""),
-        ],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(asset)void",
-        [
-            pt.Bytes(""),
-        ],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(account)void",
-        [
-            pt.Int(1),
-        ],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(uint64,uint64)void",
-        [pt.abi.String(), pt.abi.Uint64()],
-        None,
-        None,
-        pt.TealTypeError,
-    ),
-    (
-        pt.Int(1),
-        "add(uint64,uint64)void",
-        [pt.abi.Uint64()],
-        None,
-        None,
-        pt.TealInputError,
-    ),
-    (
-        pt.Int(1),
-        "add(uint64,uint64)void",
-        [pt.abi.Uint64(), pt.abi.Uint64(), pt.abi.Uint64()],
-        None,
-        None,
-        pt.TealInputError,
-    ),
-)
+    expected, _ = pt.Seq(
+        pt.InnerTxnBuilder.Begin(), expected_expr, pt.InnerTxnBuilder.Submit()
+    ).__teal__(avm6Options)
+    expected.addIncoming()
+    expected = pt.TealBlock.NormalizeBlocks(expected)
+
+    actual, _ = expr.__teal__(avm6Options)
+    actual.addIncoming()
+    actual = pt.TealBlock.NormalizeBlocks(actual)
+
+    with pt.TealComponent.Context.ignoreScratchSlotEquality(), pt.TealComponent.Context.ignoreExprEquality():
+        assert actual == expected
+        assert pt.TealBlock.MatchScratchSlotReferences(
+            pt.TealBlock.GetReferencedScratchSlots(actual),
+            pt.TealBlock.GetReferencedScratchSlots(expected),
+        )
 
 
 # txn_test.py performs additional testing
