@@ -1,3 +1,4 @@
+import pytest
 from typing import List
 
 import pyteal as pt
@@ -147,47 +148,70 @@ def __test_single_with_vars(
         assert actual == expected
 
 
-def test_multi_value():
-    ops = (
+@pytest.mark.parametrize(
+    "op",
+    [
         pt.Op.app_global_get_ex,
         pt.Op.app_local_get_ex,
         pt.Op.asset_holding_get,
         pt.Op.asset_params_get,
+    ],
+)
+@pytest.mark.parametrize(
+    "type", [pt.TealType.uint64, pt.TealType.bytes, pt.TealType.anytype]
+)
+@pytest.mark.parametrize("iargs", [[], ["AssetFrozen"]])
+@pytest.mark.parametrize("args", [[], [pt.Int(0)], [pt.Int(1), pt.Int(2)]])
+def test_multi_value(op, type, iargs, args):
+    reducer = (
+        lambda value, hasValue: pt.If(hasValue)
+        .Then(value)
+        .Else(pt.App.globalGet(pt.Bytes("None")))
     )
-    types = (pt.TealType.uint64, pt.TealType.bytes, pt.TealType.anytype)
-    immedate_argv = ([], ["AssetFrozen"])
-    argv = ([], [pt.Int(0)], [pt.Int(1), pt.Int(2)])
+    expr = pt.MultiValue(
+        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
+    )
+    __test_single_conditional(expr, op, args, iargs, reducer)
 
-    for op in ops:
-        for type in types:
-            for iargs in immedate_argv:
-                for args in argv:
-                    reducer = (
-                        lambda value, hasValue: pt.If(hasValue)
-                        .Then(value)
-                        .Else(pt.App.globalGet(pt.Bytes("None")))
-                    )
-                    expr = pt.MultiValue(
-                        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
-                    )
-                    __test_single_conditional(expr, op, args, iargs, reducer)
+    reducer = lambda value, hasValue: pt.Seq(pt.Assert(hasValue), value)  # noqa: E731
+    expr = pt.MultiValue(
+        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
+    )
+    __test_single_assert(expr, op, args, iargs, reducer)
 
-                    reducer = lambda value, hasValue: pt.Seq(  # noqa: E731
-                        pt.Assert(hasValue), value
-                    )
-                    expr = pt.MultiValue(
-                        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
-                    )
-                    __test_single_assert(expr, op, args, iargs, reducer)
+    hasValueVar = pt.ScratchVar(pt.TealType.uint64)
+    valueVar = pt.ScratchVar(type)
+    reducer = lambda value, hasValue: pt.Seq(  # noqa: E731
+        hasValueVar.store(hasValue), valueVar.store(value)
+    )
+    expr = pt.MultiValue(
+        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
+    )
+    __test_single_with_vars(expr, op, args, iargs, hasValueVar, valueVar, reducer)
 
-                    hasValueVar = pt.ScratchVar(pt.TealType.uint64)
-                    valueVar = pt.ScratchVar(type)
-                    reducer = lambda value, hasValue: pt.Seq(  # noqa: E731
-                        hasValueVar.store(hasValue), valueVar.store(value)
-                    )
-                    expr = pt.MultiValue(
-                        op, [type, pt.TealType.uint64], immediate_args=iargs, args=args
-                    )
-                    __test_single_with_vars(
-                        expr, op, args, iargs, hasValueVar, valueVar, reducer
-                    )
+
+def test_multi_compile_check():
+    def never_fails(options):
+        return
+
+    program_never_fails = pt.MultiValue(
+        pt.Op.app_global_get_ex,
+        [pt.TealType.uint64, pt.TealType.uint64],
+        compile_check=never_fails,
+    )
+    program_never_fails.__teal__(options)
+
+    class TestException(Exception):
+        pass
+
+    def always_fails(options):
+        raise TestException()
+
+    program_always_fails = pt.MultiValue(
+        pt.Op.app_global_get_ex,
+        [pt.TealType.uint64, pt.TealType.uint64],
+        compile_check=always_fails,
+    )
+
+    with pytest.raises(TestException):
+        program_always_fails.__teal__(options)
