@@ -1,8 +1,8 @@
-from ..errors import TealInputError
-from ..types import TealType, require_type
+from pyteal.errors import TealInputError
+from pyteal.types import TealType, require_type
 
-from .expr import Expr
-from .scratch import ScratchSlot, ScratchLoad, ScratchStore
+from pyteal.ast.expr import Expr
+from pyteal.ast.scratch import ScratchSlot, ScratchLoad, ScratchStore
 
 
 class ScratchVar:
